@@ -351,13 +351,14 @@ transform_schema = Schema({
 
 postprocess_schema = Schema({
     Optional('LabelShift'):  int,
-    Optional('SquadV1'): {
-        'label_file': str,
-        'vocab_file': str
-    },
     Optional('Collect'): {
         'length': int
     },
+    Optional('SquadV1'): {
+        'label_file': str,
+        'vocab_file': str,
+        Optional('do_lower_case', default='True'): bool
+    },
 })
 
 dataset_schema = Schema({
